--
-- File : field_mapping.lua
-- Date : 24 Jul 2012
-- Updated by : W Rogers
--
-- Notes:
--		17 July 2012 :	Contains update shipped with v1.23 plus updates from 3 July
--						Fix in activity priority also included
--		20 July 2012 :  Updated based on feedback from BK
--						Reviewed all integer to real64 bit field maps
--						Reviewed all date mappings
--                      Includes the PUTCALLIND mappings exposed
--						The QoS erroneous mappings removed
--						LIFETIME Hi/lo mappings now to both LOC "life of contract" and Life_hi/lo fields
--						TS (time seconds) timestamps removed.
--						Note: fields compatible with RDMFieldDictionary 4.00.14 release date 18-Nov-2010
--
--		24 July 2012 :  Removed erroneous inclusion of ACTIVITY.DATE, TRADE.DATE, QUOTE.DATE fields
--						These fields do not appear in the CSP output at this time.
--						Fixed mapping of BID.LOW, no function needed.
--						Removed erroneous auto-generated mapping to PUTCALLIND field.
--						Corrected TRDPRC_1 misspelling
--		30 July	2012 :  SEC_ACT_1 field now associated with SC_ACT_TP1 as a qualifier field
--						SHRS.OUTSTANDING now also mapped to AMT_OS, continue to populate AMT_ISSUE
--						"YEST.NAV.PRICE" now also mapped to NA_VALUE1, to conform to NASDAQ information
--
--						NOTE: Adjusted, in error, some Date fields transforms - they need investigation on
--						source by source basis, some are in yyyymmdd as integer values, and if converted to
--						unix time go back to 1971.
--		22 Aug 2012 :	TRADE.INDIC.VOL mapped to IND_AUCVOL
--						TRADE.INDIC.PRICE mapped to IND_AUC
--						TRADE.INDIC.OPEN mapped to THEO_OPEN  - override where necessary
--						INDEX.INDIC.PRICE mapped to THEO_OPEN  - override where necessary
--						See release notes for full additions to spreadsheet.
--		23 Aug 2012 : 	State Street uses HST_CLSBID/ASK to reflect closing quotes of the normal trading session.
--						The specific mappings for State Street use IDC.FINAL.BID / ASK to hold those values.
--						Logical mappings of YEST.BID.CLOSE and YEST.ASK.CLOSE have been removed.
--						In the futures we will create negative FIDs that explicitly give State St. what they want 
-- 						and will use YEST.BID.CLOSE to map to HST_CLSBID/ASK values.
--
-- Copyright IDC 2012
--
--
-- array used for flattening
flatten_ask = {
    "ASK",
    "BEST_ASK1",
    "BEST_ASK2",
    "BEST_ASK3",
    "BEST_ASK4",
    "BEST_ASK5",
    "BEST_ASK6",
    "BEST_ASK7",
    "BEST_ASK8",
    "BEST_ASK9",
    "BEST_ASK10",
}

flatten_bid = {
    "BID",
    "BEST_BID1",
    "BEST_BID2",
    "BEST_BID3",
    "BEST_BID4",
    "BEST_BID5",
    "BEST_BID6",
    "BEST_BID7",
    "BEST_BID8",
    "BEST_BID9",
    "BEST_BID10",
}

flatten_ask_size = {
    "ASKSIZE",
    "BEST_ASIZ1",
    "BEST_ASIZ2",
    "BEST_ASIZ3",
    "BEST_ASIZ4",
    "BEST_ASIZ5",
    "BEST_ASIZ6",
    "BEST_ASIZ7",
    "BEST_ASIZ8",
    "BEST_ASIZ9",
    "BEST_ASZ10",
}

flatten_bid_size = {
    "BIDSIZE",
    "BEST_BSIZ1",
    "BEST_BSIZ2",
    "BEST_BSIZ3",
    "BEST_BSIZ4",
    "BEST_BSIZ5",
    "BEST_BSIZ6",
    "BEST_BSIZ7",
    "BEST_BSIZ8",
    "BEST_BSIZ9",
    "BEST_BSZ10",
}

flatten_ask_time = {
    "B_ASK1_TIM",
    "B_ASK2_TIM",
    "B_ASK3_TIM",
    "B_ASK4_TIM",
    "B_ASK5_TIM",
}

flatten_bid_time = {
    "B_BID1_TIM",
    "B_BID2_TIM",
    "B_BID3_TIM",
    "B_BID4_TIM",
    "B_BID5_TIM",
}

config {
    -- this is the default field mapping, everything else overrides entries
    -- from the default
    activity_fields {
        field("PRIMACT_1", "ACT_TP_1"),
        field("SEC_ACT_1", "SC_ACT_TP1"),
    },

    -- determines the order.. which becomes first, second etc.
    activity_priority { 4, 40, 23, 24 },

    field_mapping {
        -- IDC QoS fields - requested by BK, added 2011 06 05 ND
		-- ["IDLE.COUNT"] = "REC_COUNT",
		-- ["LOST.BLOCK.COUNT"] = "BLKCOUNT",
		-- ["RECORD.STALE.IND"] = "REF_COUNT",
		-- ["MSG.COUNT"] = "SEQNUM",
        -- end of IDC QoS
        --
        --[[
        -- DELAY.MINS      = "ACTIV_DELAY"
        -- (ACTIV_DELAY isn't in the sis2rfa field map)

        -- ["QUOTE.COND_1"]       = "GEN_TEXT16",
        -- needs integer_to_string
        --]]
		["ASK.PRICE"] = activity("ASK", 24),
		["BID.PRICE"] = activity("BID", 23),
		["TRADE.CLOSE"] = activity("SETTLE", 4),

        ["ENUM.CURRENCY"]      =
            field_octal_enum_def_lookup("CURRENCY", {
                GBp = 'GBX',
                ILs = 'ILS',   -- (SR: 1250896) [REF:1003864919960]
                ZAc = 'ZAC',   -- (SR: 1250896) [REF:1003864919960]
            }),

		["SHRS.OUTSTANDING"] =
				many{integer_to_real64("AMT_ISSUE"),
                     integer_to_real64("AMT_OS")},

		["VWAP"] = "VWAP",
		["TRADE.VOL"] = integer_to_real64("ACVOL_1"),
		["TRADE.PRICE"] = "TRDPRC_1",
        ["TRADE.INDIC.PRICE"]  = "IND_AUC", -- another alias
        -- ["CURRENT.PRICE"]      = "TRDPRC_1", -- removed 2010_11_17
		["INSTR_NAME2"] = "DSPLY_NAME",

        -- ["ASK.INDIC.PRICE"]    = "IMP_ASK",
        -- ["BID.INDIC.PRICE"]    = "IMP_BID",
		["TRADE.SIZE"] = integer_to_real64("TRDVOL_1"),
		["TRADE.INDIC.SIZE"] = many { integer_to_real64("TRDVOL_1"),
									  integer_to_real64("IND_AUCVOL") },
		["ASK.SIZE"] = integer_to_real64("ASKSIZE"),
		["MKT.ORDER.ASK.SIZE"] = integer_to_real64("MKOASK_VOL"),
		["ASK.INDIC.SIZE"] = integer_to_real64("ASKSIZE"),
		-- ["ASK.INDIC.SIZE"] = many { integer_to_real64("ASKSIZE"),
		--							integer_to_real64("IMP_ASIZE") },
        -- ["ASK.COUNT"]          = integer_to_real64("ASKSIZE"), -- another alias?
        --                        Removed July 2011 - bad mapping "
		["BID.SIZE"] = integer_to_real64("BIDSIZE"),
		["MKT.ORDER.BID.SIZE"] = integer_to_real64("MKOBID_VOL"),
		["BID.INDIC.SIZE"] = integer_to_real64("BIDSIZE"),
		-- ["BID.INDIC.SIZE"] = many { integer_to_real64("BIDSIZE"),
		--							integer_to_real64("IMP_BSIZE") },
		-- ["BID.COUNT"]          = integer_to_real64("BIDSIZE""), -- another alias?
        --                        Removed July 2011 - bad mapping "
		["PREV.TRADE.DATE"] = "HSTCLSDATE",
		["SETTLE.DATE"] = "SETTLEDATE",
		["TRADE.HIGH"] = "HIGH_1",
		["TRADE.LOW"] = "LOW_1",

		["CHG"] = "NETCHNG_1",
		["PCT.CHG"] = "PCTCHNG",
		["SETTLE.PRICE"] = "SETTLE",
		["STRIKE.PRICE"] = "STRIKE_PRC",
		["TRADE.OPEN"] = "OPEN_PRC",
		-- Added April 2011 for Tel Aviv (266) - ND
        -- Ammended Jun 2011 to map to both fields - ND"
		["THEORETICAL.OPEN"] = many{"AUCTIONPRC", "THEO_OPEN"},





		["INSTR_NAME"] = "DSPLY_NAME",
		["OPEN_INT"] = integer_to_real64("OPINT_1"),

		["BID.CLOSE"] = "CLOSE_BID",
		["ASK.CLOSE"] = "CLOSE_ASK",
        -- where does this go?
        -- ["CONTRIBUTOR.ID"]     = ??

        -- special
        -- date/time types
		["ACTIVITY.DATETIME"] = field_date_time("ACTIV_DATE", "TIMACT"),
		["TRADE.DATETIME"] = field_date_time("TRADE_DATE", "TRDTIM_1"),
		["FIRST.MATURITY.DATE"] = field_date_yyyymmdd("MATUR_DATE"),

		["MATURITY.DATE"] = field_date_yyyymmdd("MATUR_DATE"),
		["EXPIRATION.DATE"] = field_date_yyyymmdd("EXPIR_DATE"),

		["ISIN"] = "ISIN_CODE",
		["SEDOL"] = "SEDOL",
		["CUSIP"] = "CUSIP_CD",

		["BID.YIELD"] = "BID_YIELD",
		["YIELD"] = "YIELD",
		["ASK.YIELD"] = "ASK_YIELD",

        -- IDC.FINAL fields added Nov 2010 - ND
		["IDC.FINAL.BID"] = "HST_CLSBID",
		["IDC.FINAL.BID.DATETIME"] = "HSTCLBDDAT",
		["IDC.FINAL.ASK"] = "HST_CLSASK",
		["IDC.FINAL.ASK.DATETIME"] = "HSTCLAKDAT",
		["IDC.FINAL.TRADE"] = "HST_CLOSE2",
		["IDC.FINAL.TRADE.DATETIME"] = "HSTCL2_DAT",
--		["YEST.BID.CLOSE"] = many{"HST_CLSBID", "CLOSE3_BID"},
--		["YEST.ASK.CLOSE"] = many{"HST_CLSASK", "CLOSE3_ASK"},
		["YEST.BID.CLOSE"] = "CLOSE3_BID",
		["YEST.ASK.CLOSE"] = "CLOSE3_ASK",

		["YEST.NAV.PRICE"] = "HST_NAV",
		["YEST.TRADE.CLOSE"] = activity("HST_CLOSE", 40),
		["YEST.TRADE.VOL"] = integer_to_real64("HST_VOL"),

		-- pass it right through, in future check it is between 0 and 7
        -- force as it is marked as a string type (even though it is sent
        -- as a number)
		["DISPLAY.PRECISION"] = field_enum_force("TRD_UNITS"),
        ["TRADE.TICK"]         = field_enum("PRCTCK_1", {
            ["+"] = 1,
            ["-"] = 2,
        }),

		["ENUM.INSTR.TYPE"]    = field_enum("GN_TX20_1", {
            [256]   = "EQUITY",
            [257]   = "COMMON_STOCK",
            [258]   = "PREFERRED_STOCK",
            [260]   = "WARRANT",
            [264]   = "PREMIUM",
            [268]   = "TRUST",
            [270]   = "RIGHT",
            [271]   = "WARRANT_RIGHT",
            [512]   = "INDEX",
            [1024]  = "FUTURE",
            [1025]  = "FUTURE_SPREAD",
            [2048]  = "OPTION",
            [2304]  = "EQUITY_OPTION",
            [2560]  = "INDEX_OPTION",
            [3072]  = "FUTURE_OPTION",
            [4096]  = "FIXED_INCOME",
            [4097]  = "BOND",
            [4098]  = "CONVERTIBLE_BOND",
            [4099]  = "MBS",
            [4100]  = "GOV_BOND",
            [4104]  = "CORP_BOND",
            [4112]  = "US_AGENCY_BOND",
            [4128]  = "US_TREASURY_BILL",
            [4160]  = "US_TREASURY_COUPON",
            [4224]  = "MONEY_MARKET",
            [4352]  = "CD",
            [8192]  = "MUTUAL_FUND",
            [8193]  = "EXCHANGE_TRADED_FUND",
            [16384] = "FOREX",
            [16385] = "FOREX_FRA",
            [16386] = "FOREX_DEPOSIT",
            [17408] = "FOREX_FORWARD",
            [20480] = "STATISTICS",
            [65534] = "TEST",
        }),

        level_fields {
            ["QUOTE.DATETIME"]      = field_date_time("QUOTE_DATE", "QUOTIM"),
            ["ASK.DEPTH.PRICE"]   = "ASK",
            ["ASK.LEVEL.PRICE"]   = "ASK",
            ["ASK.PART.PRICE"]    = "ASK",
            ["ASK.DEPTH.SIZE"]    = integer_to_real64("ASKSIZE"),
            ["ASK.LEVEL.SIZE"]    = integer_to_real64("ASKSIZE"),
            ["ASK.PART.SIZE"]     = integer_to_real64("ASKSIZE"),
            ["BID.DEPTH.PRICE"]   = "BID",
            ["BID.LEVEL.PRICE"]   = "BID",
            ["BID.PART.PRICE"]    = "BID",
            ["BID.DEPTH.SIZE"]    = integer_to_real64("BIDSIZE"),
            ["BID.LEVEL.SIZE"]    = integer_to_real64("BIDSIZE"),
            ["BID.PART.SIZE"]     = integer_to_real64("BIDSIZE"),
        },

        flatten_fields {
            ["QUOTE.DATETIME"]    = field_date_time({ "QUOTE_DATE" }, { "QUOTIM" }),
            ["ASK.DEPTH.PRICE"]   = flatten_ask,
            ["ASK.LEVEL.PRICE"]   = flatten_ask,
            ["ASK.PART.PRICE"]    = flatten_ask,
            ["ASK.DEPTH.SIZE"]    = integer_to_real64(flatten_ask_size),
            ["ASK.LEVEL.SIZE"]    = integer_to_real64(flatten_ask_size),
            ["ASK.PART.SIZE"]     = integer_to_real64(flatten_ask_size),
            ["BID.DEPTH.PRICE"]   = flatten_bid,
            ["BID.LEVEL.PRICE"]   = flatten_bid,
            ["BID.PART.PRICE"]    = flatten_bid,
            ["BID.DEPTH.SIZE"]    = integer_to_real64(flatten_bid_size),
            ["BID.LEVEL.SIZE"]    = integer_to_real64(flatten_bid_size),
            ["BID.PART.SIZE"]     = integer_to_real64(flatten_bid_size),
        },

        -- populate put/call indicator
        -- first filter month out of symbol, then use that to populate the
        -- indicator field. Have to double escape backslashes in regular
        -- expressions due to lua.
        ["SYMBOL.TICKER"] = many {
            "PROV_SYMB",
            field_filter_enum(
                "PUTCALLIND", "([A-Z])\\\\?\\d+?\\\\?\\d+(?:\\.\\d+)?$", {
                A = 0, B = 0, C = 0, D = 0, E = 0, F = 0,
                G = 0, H = 0, I = 0, J = 0, K = 0, L = 0,
                M = 1, N = 1, O = 1, P = 1, Q = 1, R = 1,
                S = 1, T = 1, U = 1, V = 1, W = 1, X = 1, }),
        },
        -- please note that the mapping below will need to be added as a "many" to
        -- the above put/call mapping if you require it to be uncommented

		-- ["SYMBOL.TICKER"] = "PROV_SYMB",

        ["PUT.CALL.IND"]       = field_enum("PUTCALLIND", { C = 0, P = 1 }),
		["SYMBOL.EXCH.TICKER"] = "MNEMONIC",

		["MKT.SEGMENT"] = field_octal_string("MKT_SEGMNT"),

		["FUND.RECORD.STATUS"] = "HM_STATUS",






		["QUOTE.DATETIME"] = field_date_time("QUOTE_DATE", "QUOTIM"),

		-- ["OFFICIAL.VWAP"] = "VWAP",
		["LATE.TRADE.DATE"] = field_date_yyyymmdd("PDTRDDATE"),

		["BID.HIGH"] = "BID_HIGH_1",
		["BID.LOW"] = "BID_LOW_1",

		["BID.CHG"] = "BID_NET_CH",
		["INDIC.NAV.PRICE"] = "IND_NAV",
		["RECORD.DATE"] = "REC_DATE",

		["CORRECTION.NEW.TRADE.PRICE"] = "INSPRC",
		["CORRECTION.NEW.TRADE.SIZE"] = integer_to_real64("INSVOL"),

		["CORRECTION.PREV.TRADE.SIZE"] = integer_to_real64("IRGVOL"),

		["COUPON.PRICE"] = "COUPN_RATE",
		["LIFETIME.HIGH_DATE"] = "LF_HGH_DAT",
	---	["ENUM.CURRENCY"] = "CURRENCY",

		["ORDERBOOK.TRADE.VOL"] = integer_to_real64("ORDBK_VOL"),
		["OFF_BOOK.TRADE.VOL"] = integer_to_real64("OFFBK_VOL"),
		["ORDERBOOK.TURNOVER"] = integer_to_real64("TURN_ORDB"),
		["OPEN.INT.DATE"] = field_date_yyyymmdd("OPINT_DATE"),

		["TRADE.INDIC.OPEN"] = "THEO_OPEN",
		["TRADE.INDIC.VOL"] = integer_to_real64("IND_AUCVOL"),
		["LIMIT.HIGH"] = "UPLIMIT",
		["LIMIT.LOW"] = "LOLIMIT",
		["MID.HIGH"] = "MID_HIGH",
		["MID.LOW"] = "MID_LOW",
		["MID.OPEN"] = "MID_OPEN",
		["MID.PRICE"] = "MID_PRICE",
		["MKT.NORMAL.SIZE"] = integer_to_real64("NML_MKT_SZ"),

		["NAV.PRICE"] = "NAVALUE",
		["STATIC.REF.PRICE"] = "REF_PRICE",

		["TRADE.OFFICIAL.CLOSE"] = "OFF_CLOSE",
		["FIRST.RANGE.OPEN"] = "OPEN1",
		["SECOND.RANGE.OPEN"] = "OPEN2",
		["TRADE.BLOCK.COUNT"] = integer_to_real64("BLKCOUNT"),
		["TRADE.BLOCK.VOL"] = integer_to_real64("BLKVOLUM"),
		["TRADE.COUNT"] = integer_to_real64("NUM_MOVES"),

		["TRADE.COND.PRICE"] = "IRGPRC",
		["TRADE.COND.SIZE"] = integer_to_real64("IRGVOL"),


		["TRADE.TOTAL.VALUE"] = integer_to_real64("TURNOVER"),

		["LIFETIME.LOW_DATE"] = "LF_LOW_DAT",






		["TRADE.AUTO.VWAP"] = "ORDBK_VWAP",

		["AUCTION.PRICE"] = "AUCTIONPRC",
		["LATE.TRADE.PRICE"] = "PDTRDPRC",
		["AUCTION.VOL"] = integer_to_real64("AUCTIONVOL"),
		["LATE.TRADE.SIZE"] = integer_to_real64("PREDAYVOL"),
		["BID.CLOSE_SIZE"] = integer_to_real64("CLS_BIDSIZ"),
		["ASK.CLOSE_SIZE"] = integer_to_real64("CLS_ASKSIZ"),
		["ISSUE.DATE"] = "ISSUE_DATE",
		["LOT.SIZE"] = integer_to_real64("LOT_SIZE"),
		["INDEX.INDIC.PRICE"] = "THEO_OPEN",
		["SECOND.RANGE.CLOSE"] = "CLOSE2",
		["FIRST.RANGE.CLOSE"] = "CLOSE1",





		["MID.SPREAD_TO_MATURITY"] = "MID_SPREAD",
		["BID.SPREAD"] = "BID_SPREAD",
		["ASK.SPREAD"] = "ASK_SPREAD",
		["IMPLIED.VOLATILITY"] = "IMP_VOLT",




		["INDIC.AUCTION.VOL"] = integer_to_real64("IND_AUCVOL"),
		["INDIC.AUCTION.PRICE"] = "IND_AUC",


		["HISTORICAL.TRADE.SIZE"] = integer_to_real64("PREDAYVOL"),
		["HISTORICAL.TRADE.PRICE"] = "PDTRDPRC",




		["UPPER.PRICE.BAND"] = "UPLIMIT",
		["LOWER.PRICE.BAND"] = "LOLIMIT",

		["STATIC.PRICEBAND.PCT"] = "HL_PCT_FL",
		["ACCUMULATED.TURNOVER"] = "TURNOVER",
		["BASIS.PRICE"] = "BASE_PRC1",
		["BASIS.HIGH.LIMIT"] = "UPLIMIT",
		["BASIS.LOW.LIMIT"] = "LOLIMIT",




		["PRIOR.SESSION.SETTLE.PRICE"] = "PRE_SETTLE",
		["PRIOR.SESSION.TRADE.HIGH"] = "PRV_HIGH",
		["PRIOR.SESSION.TRADE.LOW"] = "PRV_LOW",
		["PRIOR.SESSION.TRADE.OPEN"] = "PRV_OPEN",
		["PRIOR.SESSION.YEST.TRADE.CLOSE"] = "PRV_LAST",
		["TRADE.OAS.SPREAD"] = "OAS",
		["BID.OAS.SPREAD"] = "OAS_BID",






		["NUM.ADVANCES"] = integer_to_real64("ISSUES_ADV"),
		["NUM.DECLINES"] = integer_to_real64("ISSUES_DEC"),
		["NUM.UNCHANGED"] = integer_to_real64("ISSUES_UNC"),
		["IDC.TURNOVER"] = "TURNOVER",


		["30DAY.MEAN.VOL"] = integer_to_real64("VMA_30D"),
		["FUND.BETA"] = "BETA_VAL",




		["LIFETIME.HIGH"] = many{"LIFE_HIGH", "LOCHIGH"},
		["ERN.CURRENTYR.PS.PRICE"] = "EARNINGS",


--		["EFFECTIVE.DATE"] = "CP_EFF_DAT",
		["DIV.EX_DATE"] = "EXDIVDATE",
		["DIV.PRICE"] = "DIVIDEND",
		["DIV.PAY_DATE"] = "DIVPAYDATE",
		["RATIO.PE"] = "PERATIO",
		["WEEK52.TRADE.HIGH"] = "52WK_HIGH",
		["WEEK52.TRADE.LOW"] = "52WK_LOW",
		["LIFETIME.LOW"] = many{"LIFE_LOW", "LOCLOW"},

		["DIV.YIELD"] = "YIELD",
		["YEST.YIELD"] = "HST_CLSYLD",

		["YR1.DIV.PRICE"] = "DPS1_1",
		["YR2.DIV.PRICE"] = "DPS1_2",
		["YR3.DIV.PRICE"] = "DPS1_3",
		["YR4.DIV.PRICE"] = "DPS1_4",

		["FUND.52WEEK.TRADE.HIGH_DATE"] = "52W_HDAT",
		["FUND.52WEEK.TRADE.LOW_DATE"] = "52W_LDAT",
		["HISTORICAL.VOLATILITY.VALUE"] = "HST_VOLT",
		["UNDERLYING.STOCK.PRICE"] = "UNDLY1_PRC",
		["MIC.CODE"] = "MIC_CODE",

		["BASE.PRICE"] = "BASE_PRC1",

		["DELISTING.DATE"] = "DELIST_DAT",
		["NOMINAL.VALUE"] = "NOMINAL",
		["BLOCK.TRADE.CRITERIA.SIZE"] = integer_to_real64("BLKUNIT"),

        -- ["DEPTH.KEY"]    = key for this field type
        -- ["ASK.DEPTH.COUNT"]    = level_field(),
        -- ["BID.DEPTH.COUNT"]    = level_field(),
        -- ["TRADE.DEPTH.PHASE"]    = level_field(),
        -- ["BID.DEPTH.DATETIME"]    = level_field(),
        -- ["ASK.DEPTH.DATETIME"]    = level_field(),
        -- ["BID.DEPTH.ID"]    = level_field(),
        -- ["ASK.DEPTH.ID"]    = level_field(),
        -- ["BID.DEPTH.YIELD"]    = level_field(),
        -- ["ASK.DEPTH.YIELD"]    = level_field(),
        -- ["CED.DEPTH.FLAG"]    = level_field(),
        -- ["CED.DEPTH.INDICATOR"]    = level_field(),
        -- ["CED.DEPTH.KEY"]    = level_field(),
        -- ["XS.DEPTH.RATE"]    = level_field(),
        -- ["XS.DEPTH.RECORD.COUNT"]    = level_field(),
        -- ASK.LEVEL                                = 278,
        -- BID.LEVEL                                = 279,
        -- DIV.LEVEL                                = 286,
        -- PRICE.LEVEL                              = 393,
        -- BID.LEVEL.SIZE                           = 609,
        -- PRICE.LEVEL.BID_COUNT                    = 610,
        -- BID.LEVEL.PRICE                          = 612,
        -- PRICE.LEVEL.ASK_COUNT                    = 615,
        -- BID.LEVEL.COUNT                          = 616,
        -- ASK.LEVEL.COUNT                          = 617,
        -- BID.ASK.LEVEL                            = 699,
        -- INSTRUMENT.LEVEL                         = 3765,
        -- ORDERBOOK.LEVEL                          = 3766,
        -- SECTOR.LEVEL                             = 3788,

        -- UNKNOWN fields
        -- integer mapped to date errors

        -- this one I'm really not sure about
        -- COUPON.RATE sis: COUPON_RATE         = ?

        -- these are values no in ctf/codes.common.cf and therefore unchecked
        -- the name on the left of each of these is it's SIS identifier
        -- ACTIV_PRICE_1          = "PRIMACT_1",
        -- ACTIV_PRICE_2          = "SEC_ACT_1",
        -- ACTIV_TYPE_1           = "ACT_TP_1",
        -- ACTIV_TYPE_2           = "SC_ACT_TP1",
        -- ASK_EXCH               = "ASK_MMID1",
        -- ASK_TONE               = "ASK_TONE",
        -- BID_EXCH               = "BID_MMID1",
        -- BID_TONE               = "BID_TONE",
        -- CONTRACT_MONTH         = "CONTR_MNTH",
        -- DIV_AMOUNT             = "DIVIDEND",
        -- DIV_DATE_EX            = "EXDIVDATE",
        -- DIV_DATE_PAY           = "DIVPAYDATE",
        -- DIV_YIELD              = "YIELD",
        -- EPS_CURRENT            = "EARNINGS",
        -- EXCH_NAME              = "RDN_EXCHID",
        -- INSTRUMENT_STATUS      = "PRC_QL2",
        -- MID_PRICE              = "MID_PRICE",
        -- OPTION_TYPE            = "PUTCALLIND",
        -- PE_RATIO               = "PERATIO",
        -- PROD_CATG              = "PROD_PERM",
        -- QUOTE_CONDITION        = "PRC_QL_CD",
        -- SALE_CONDITION         = "CTS_QUAL",
        -- SHARES_ISSUED          = "AMT_ISSUE",
        -- TEMPLATE_NO            = "RDNDISPLAY",
        -- TEMPLATE_NO_EX         = "PREF_DISP",
        -- TRADE_COUNT_BLOCK      = "BLKCOUNT",
        -- TRADE_COUNT_TOTAL      = "NUM_MOVES",
        -- TRADE_EXCH_LAST        = "TRDXID_1",
        -- TRADE_HIGH_52WK        = "YRHIGH",
        -- TRADE_HIGH_52WK_DATE   = "YRHIGHDAT",
        -- TRADE_LOW_52WK         = "YRLOW",
        -- TRADE_LOW_52WK_DATE    = "YRLOWDAT",
        -- TRADE_VOL_BLOCK        = "BLKVOLUM",
    },

    defaults {
        top_of_book_marker  = 'TOP_BOOK_MARKER',
        level_field         = 'ORDER.POSITION',
        order_field         = 'ORDER.ID',
        next_order_field    = 'NEXT.ORDER.ID',
        prev_order_field    = 'PREV.ORDER.ID'
    },
}

